# Generated by Django 3.0.6 on 2020-06-24 14:40

import django.contrib.gis.db.models.fields
import django.contrib.postgres.indexes
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0036_auto_20200116_0926'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='feature',
            name='geostore_fe_identif_f6c552_idx',
        ),
        migrations.AlterField(
            model_name='featureextrageom',
            name='geom',
            field=django.contrib.gis.db.models.fields.GeometryField(spatial_index=False, srid=4326),
        ),
        migrations.AddIndex(
            model_name='feature',
            index=models.Index(fields=['layer', 'identifier'], name='geostore_fe_layer_i_141c81_idx'),
        ),
        migrations.AddIndex(
            model_name='feature',
            index=django.contrib.postgres.indexes.GinIndex(fields=['properties'], name='properties_gin_index'),
        ),
        migrations.AddIndex(
            model_name='featureextrageom',
            index=models.Index(fields=['layer_extra_geom', 'identifier'], name='geostore_fe_layer_e_aa3cea_idx'),
        ),
        migrations.AddIndex(
            model_name='featureextrageom',
            index=django.contrib.postgres.indexes.GistIndex(fields=['layer_extra_geom', 'geom'], name='feg_geom_gist_index'),
        ),
        migrations.AddIndex(
            model_name='featureextrageom',
            index=django.contrib.postgres.indexes.GinIndex(fields=['properties'], name='feg_properties_gin_index'),
        ),
    ]
