# Generated by Django 3.0.2 on 2020-01-31 17:58

import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0038_auto_20200128_1442'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='feature',
            index=django.contrib.postgres.indexes.GinIndex(fields=['properties'], name='geostore_fe_propert_46222e_gin'),
        ),
    ]
